import {
  Box,
  Button,
  Paper,
  Typography,
  Menu,
  MenuItem,
  TextField,
} from "@mui/material";
import {
  BackButton,
  editSightStore,
  languageStore,
  SelectArticleModal,
  SelectMediaDialog,
  TabPanel,
  UploadMediaDialog,
} from "@shared";
import {
  LanguageSwitcher,
  MediaArea,
  MediaAreaForSight,
  ReactMarkdownComponent,
  ReactMarkdownEditor,
} from "@widgets";
import { ImagePlus, Plus, Save, Trash2, Unlink, X } from "lucide-react";
import { observer } from "mobx-react-lite";
import { useEffect, useState } from "react";
import { toast } from "react-toastify";
import { MediaViewer } from "../../MediaViewer/index";

export const RightWidgetTab = observer(
  ({ value, index }: { value: number; index: number }) => {
    const [anchorEl, setAnchorEl] = useState<null | HTMLElement>(null);

    const {
      sight,
      updateRightArticleInfo,
      getRightArticles,
      updateSight,
      unlinkPreviewMedia,
      linkPreviewMedia,
      unlinkRightArticle,
      deleteRightArticle,
      linkArticle,
      deleteRightArticleMedia,
      createLinkWithRightArticle,
      setFileToUpload,
      createNewRightArticle,
    } = editSightStore;

    const [previewMedia, setPreviewMedia] = useState<any | null>(null);

    useEffect(() => {
      if (sight.common.preview_media) {
        setPreviewMedia(sight.common.preview_media);
      }
    }, [sight.common.preview_media]);

    const handleUnlinkPreviewMedia = () => {
      unlinkPreviewMedia();
      setPreviewMedia(null);
    };

    const [uploadMediaOpen, setUploadMediaOpen] = useState(false);
    const { language } = languageStore;
    const [type, setType] = useState<"article" | "media">("media");
    useEffect(() => {
      const fetchData = async () => {
        if (sight.common.id) {
          await getRightArticles(sight.common.id);
        }
      };
      fetchData();
      console.log(sight[language].right);
    }, [sight.common.id]);

    const [activeArticleIndex, setActiveArticleIndex] = useState<number | null>(
      null
    );
    const [isSelectModalOpen, setIsSelectModalOpen] = useState(false);
    const [isSelectMediaModalOpen, setIsSelectMediaModalOpen] = useState(false);
    const open = Boolean(anchorEl);

    const handleClick = (event: React.MouseEvent<HTMLButtonElement>) => {
      setAnchorEl(event.currentTarget);
    };
    const handleClose = () => {
      setAnchorEl(null);
    };

    const handleSelectArticle = (index: number) => {
      setActiveArticleIndex(index);
    };

    const handleCreateNew = () => {
      createNewRightArticle();
      handleClose();
    };

    const handleSelectExisting = () => {
      setIsSelectModalOpen(true);
      handleClose();
    };

    const handleCloseSelectModal = () => {
      setIsSelectModalOpen(false);
    };

    const handleArticleSelect = (id: number) => {
      linkArticle(id);
      handleCloseSelectModal();
    };

    const handleMediaSelected = async (media: {
      id: string;
      filename: string;
      media_name?: string;
      media_type: number;
    }) => {
      await createLinkWithRightArticle(
        media,
        sight[language].right[activeArticleIndex || 0].id
      );
    };

    const handleSave = async () => {
      await updateSight();
      toast.success("Достопримечательность сохранена");
    };

    useEffect(() => {
      console.log(sight[language].right);
    }, [sight[language].right]);

    return (
      <TabPanel value={value} index={index}>
        <LanguageSwitcher />
        <Box
          sx={{
            display: "flex",
            flexDirection: "column",
            height: "100%",
            minHeight: "calc(100vh - 200px)",
            gap: 2,
            paddingBottom: "70px",
            position: "relative",
          }}
        >
          <BackButton />

          <Box sx={{ display: "flex", flexGrow: 1, gap: 2.5 }}>
            <Box className="flex flex-col w-[75%] gap-2">
              <Box className="w-full flex gap-2">
                <Box className="relative w-[20%] h-[70vh] flex flex-col rounded-2xl overflow-y-auto gap-3 border border-gray-300 p-3">
                  <Box className="flex flex-col gap-3 max-h-[60vh] overflow-y-auto ">
                    <Box
                      onClick={() => setType("media")}
                      className="w-full  bg-green-200 p-4 rounded-2xl cursor-pointer text-sm hover:bg-gray-300 transition-all duration-300"
                    >
                      <Typography>Предпросмотр медиа</Typography>
                    </Box>
                    {sight[language].right.length > 0 &&
                      sight[language].right.map((article, index) => (
                        <Box
                          key={index}
                          className="w-full bg-gray-200 p-4 rounded-2xl text-sm cursor-pointer hover:bg-gray-300 transition-all duration-300"
                          onClick={() => {
                            handleSelectArticle(index);
                            setType("article");
                          }}
                        >
                          <Typography>{article.heading}</Typography>
                        </Box>
                      ))}
                  </Box>
                  <button
                    className="w-10 h-10 bg-blue-500 rounded-full absolute bottom-5 left-5 flex items-center justify-center"
                    onClick={handleClick}
                  >
                    <Plus size={20} color="white" />
                  </button>
                  <Menu
                    id="basic-menu"
                    anchorEl={anchorEl}
                    open={open}
                    onClose={handleClose}
                    MenuListProps={{
                      "aria-labelledby": "basic-button",
                    }}
                    sx={{ mt: 1 }}
                  >
                    <MenuItem onClick={handleCreateNew}>
                      <Typography>Создать новую</Typography>
                    </MenuItem>
                    <MenuItem onClick={handleSelectExisting}>
                      <Typography>Выбрать существующую статью</Typography>
                    </MenuItem>
                  </Menu>
                </Box>

                {type === "article" && (
                  <Box className="w-[80%] border border-gray-300 rounded-2xl p-3">
                    {activeArticleIndex !== null && (
                      <>
                        <Box className="flex justify-end gap-2 mb-3">
                          <Button
                            variant="contained"
                            color="primary"
                            startIcon={<Unlink color="white" size={18} />}
                            onClick={() => {
                              unlinkRightArticle(
                                sight[language].right[activeArticleIndex].id
                              );
                              setActiveArticleIndex(null);
                            }}
                          >
                            Открепить
                          </Button>
                          <Button
                            variant="contained"
                            color="error"
                            startIcon={<Trash2 size={18} />}
                            onClick={() => {
                              deleteRightArticle(
                                sight[language].right[activeArticleIndex].id
                              );
                              setActiveArticleIndex(null);
                            }}
                          >
                            Удалить
                          </Button>
                        </Box>
                        <Box sx={{ display: "flex", gap: 3, flexGrow: 1 }}>
                          <Box
                            sx={{
                              flex: 2,
                              display: "flex",
                              flexDirection: "column",
                              gap: 2,
                              maxHeight: "70%",
                            }}
                          >
                            <TextField
                              label="Название информации"
                              value={
                                sight[language].right[activeArticleIndex]
                                  .heading
                              }
                              onChange={(e) =>
                                updateRightArticleInfo(
                                  activeArticleIndex,
                                  language,
                                  e.target.value,
                                  sight[language].right[activeArticleIndex].body
                                )
                              }
                              variant="outlined"
                              fullWidth
                            />

                            <ReactMarkdownEditor
                              value={
                                sight[language].right[activeArticleIndex].body
                              }
                              onChange={(value) =>
                                updateRightArticleInfo(
                                  activeArticleIndex,
                                  language,
                                  sight[language].right[activeArticleIndex]
                                    .heading,
                                  value
                                )
                              }
                            />

                            <MediaArea
                              articleId={
                                sight[language].right[activeArticleIndex].id
                              }
                              mediaIds={
                                sight[language].right[activeArticleIndex].media
                              }
                              onFilesDrop={(files) => {
                                setFileToUpload(files[0]);
                                setUploadMediaOpen(true);
                              }}
                              deleteMedia={deleteRightArticleMedia}
                              setSelectMediaDialogOpen={() => {
                                setIsSelectMediaModalOpen(true);
                              }}
                            />
                          </Box>
                        </Box>
                      </>
                    )}
                  </Box>
                )}
                {type === "media" && (
                  <Box className="w-[80%] border border-gray-300 rounded-2xl relative">
                    {sight.common.preview_media && (
                      <>
                        <Box className="w-[80%] h-[70vh] border border-gray-300 rounded-2xl p-3 relative flex justify-center items-center">
                          {type === "media" && (
                            <Box className="w-[80%] border border-gray-300 rounded-2xl relative flex items-center justify-center">
                              {previewMedia && (
                                <>
                                  <Box className="absolute top-4 right-4  z-10">
                                    <button
                                      className="w-10 h-10 flex items-center justify-center z-10"
                                      onClick={handleUnlinkPreviewMedia}
                                    >
                                      <X size={20} color="red" />
                                    </button>
                                  </Box>

                                  <Box sx={{}}>
                                    <MediaViewer
                                      media={{
                                        id: previewMedia.id || "",
                                        media_type: previewMedia.media_type,
                                        filename: previewMedia.filename || "",
                                      }}
                                    />
                                  </Box>
                                </>
                              )}
                              {!previewMedia && (
                                <MediaAreaForSight
                                  onFinishUpload={(mediaId) => {
                                    linkPreviewMedia(mediaId);
                                  }}
                                  onFilesDrop={() => {}}
                                />
                              )}
                            </Box>
                          )}
                        </Box>
                      </>
                    )}

                    {!sight.common.preview_media && (
                      <Box className="w-full h-full flex justify-center items-center">
                        <Box
                          sx={{
                            maxWidth: "500px",
                            maxHeight: "100%",
                            display: "flex",
                            flexGrow: 1,
                            margin: "0 auto",
                            justifyContent: "center",
                          }}
                        >
                          <MediaAreaForSight
                            onFinishUpload={(mediaId) => {
                              linkPreviewMedia(mediaId);
                            }}
                            onFilesDrop={() => {}}
                          />
                        </Box>
                      </Box>
                    )}
                  </Box>
                )}
              </Box>
            </Box>

            {type === "article" && (
              <Box className="w-[25%] mr-10">
                {activeArticleIndex !== null && (
                  <Paper
                    className="flex-1 flex flex-col rounded-2xl"
                    elevation={2}
                  >
                    <Box
                      className="rounded-2xl overflow-hidden"
                      sx={{
                        width: "100%",
                        height: "75vh",
                        background: "#877361",
                        borderColor: "grey.300",
                        display: "flex",
                        flexDirection: "column",
                      }}
                    >
                      {sight[language].right[activeArticleIndex].media.length >
                      0 ? (
                        <MediaViewer
                          media={
                            sight[language].right[activeArticleIndex].media[0]
                          }
                        />
                      ) : (
                        <Box
                          sx={{
                            width: "100%",
                            height: 200,
                            flexShrink: 0,

                            backgroundColor: "rgba(0,0,0,0.1)",
                            display: "flex",
                            alignItems: "center",
                            justifyContent: "center",
                          }}
                        >
                          <ImagePlus size={48} color="white" />
                        </Box>
                      )}

                      <Box
                        sx={{
                          width: "100%",
                          height: "70px",
                          background: "#877361",
                          display: "flex",
                          flexShrink: 0,

                          alignItems: "center",
                          borderBottom: "1px solid rgba(255,255,255,0.1)",
                          px: 2,
                        }}
                      >
                        <Typography variant="h6" color="white">
                          {sight[language].right[activeArticleIndex].heading ||
                            "Выберите статью"}
                        </Typography>
                      </Box>

                      <Box
                        sx={{
                          px: 2,
                          flexGrow: 1,
                          flex: 1,
                          minHeight: "300px",
                          overflowY: "auto",
                          backgroundColor: "#877361",
                          color: "white",
                          py: 1,
                        }}
                      >
                        {sight[language].right[activeArticleIndex].body ? (
                          <ReactMarkdownComponent
                            value={
                              sight[language].right[activeArticleIndex].body
                            }
                          />
                        ) : (
                          <Typography
                            color="rgba(255,255,255,0.7)"
                            sx={{ textAlign: "center", mt: 4 }}
                          >
                            Предпросмотр статьи появится здесь
                          </Typography>
                        )}
                      </Box>
                    </Box>
                  </Paper>
                )}
              </Box>
            )}
          </Box>

          <Box
            sx={{
              position: "absolute",
              bottom: 0,
              right: 0,
              padding: 2,
              backgroundColor: "background.paper",
              width: "100%",
              display: "flex",
              justifyContent: "flex-end",
            }}
          >
            <Button
              variant="contained"
              startIcon={<Save color="white" size={18} />}
              color="success"
              onClick={handleSave}
            >
              Сохранить
            </Button>
          </Box>
        </Box>

        <UploadMediaDialog
          open={uploadMediaOpen}
          onClose={() => setUploadMediaOpen(false)}
          afterUpload={async (media) => {
            setUploadMediaOpen(false);
            setFileToUpload(null);
            await createLinkWithRightArticle(
              media,
              sight[language].right[activeArticleIndex || 0].id
            );
          }}
        />

        <SelectArticleModal
          open={isSelectModalOpen}
          onClose={handleCloseSelectModal}
          onSelectArticle={handleArticleSelect}
        />
        <SelectMediaDialog
          open={isSelectMediaModalOpen}
          onClose={() => setIsSelectMediaModalOpen(false)}
          onSelectMedia={handleMediaSelected}
        />
      </TabPanel>
    );
  }
);
